@@ -16,7 +16,7 @@
 #
 #  partly based on work by Nokia Solutions and Networks Oyj
 """A parallel executor for Robot Framework test cases.
-Version 2.5.5
+Version 2.7.0
 
 Supports all Robot Framework command line options and also following
 options (these must be before normal RF options):
@@ -120,6 +120,7 @@ from .execution_items import (
     SuiteItem,
     SuiteItems,
     TestItem,
+    RunnableItem,
 )
 from .result_merger import merge
 
@@ -785,7 +786,7 @@ else:
         return open(".pabotsuitenames", mode, encoding="utf-8")
 
 
-def shard_suites(suite_names, pabot_args):
+def solve_shard_suites(suite_names, pabot_args):
     if pabot_args.get("shardcount", 1) <= 1:
         return suite_names
     if "shardindex" not in pabot_args:
@@ -894,26 +895,34 @@ def _regenerate(
         and "suitesfrom" in pabot_args
         and os.path.isfile(pabot_args["suitesfrom"])
     ):
-        suites = _suites_from_outputxml(pabot_args["suitesfrom"])
-        if file_h is None or file_h.dirs != h.dirs:
-            all_suites = generate_suite_names_with_builder(
-                outs_dir, datasources, options
-            )
-        else:
-            all_suites = [suite for suite in lines if suite]
-        suites = _preserve_order(all_suites, suites)
+        suites = _suites_from_file(file_h, h, pabot_args, outs_dir, datasources, options, lines)
     else:
-        suites = _levelsplit(
-            generate_suite_names_with_builder(outs_dir, datasources, options),
-            pabot_args,
-        )
-        suites = _preserve_order(suites, [suite for suite in lines if suite])
+        suites = _suites_from_wrong_or_empty_file(pabot_args, outs_dir, datasources, options, lines)
     if suites:
         store_suite_names(h, suites)
     assert all(isinstance(s, ExecutionItem) for s in suites)
     return suites
 
 
+def _suites_from_file(file_h, h, pabot_args, outs_dir, datasources, options, lines):
+    suites = _suites_from_outputxml(pabot_args["suitesfrom"])
+    if file_h is None or file_h.dirs != h.dirs:
+        all_suites = generate_suite_names_with_builder(
+            outs_dir, datasources, options
+        )
+    else:
+        all_suites = [suite for suite in lines if suite]
+    return _preserve_order(all_suites, suites)
+
+
+def _suites_from_wrong_or_empty_file(pabot_args, outs_dir, datasources, options, lines):
+    suites = _levelsplit(
+        generate_suite_names_with_builder(outs_dir, datasources, options),
+        pabot_args,
+    )
+    return _preserve_order(suites, [suite for suite in lines if suite])
+
+
 def _contains_suite_and_test(suites):
     return any(isinstance(s, SuiteItem) for s in suites) and any(
         isinstance(t, TestItem) for t in suites
@@ -922,6 +931,8 @@ def _contains_suite_and_test(suites):
 
 def _preserve_order(new_items, old_items):
     assert all(isinstance(s, ExecutionItem) for s in new_items)
+    if not old_items:
+        return new_items
     assert all(isinstance(s, ExecutionItem) for s in old_items)
     old_contains_tests = any(isinstance(t, TestItem) for t in old_items)
     old_contains_suites = any(isinstance(s, SuiteItem) for s in old_items)
@@ -1631,7 +1642,7 @@ class QueueItem(object):
 
 
 def _create_execution_items(
-    suite_names, datasources, outs_dir, options, opts_for_run, pabot_args
+    suite_groups, datasources, outs_dir, options, opts_for_run, pabot_args
 ):
     is_dry_run = (
         options.get("dryrun")
@@ -1640,11 +1651,11 @@ def _create_execution_items(
     )
     if is_dry_run:
         all_items = _create_execution_items_for_dry_run(
-            suite_names, datasources, outs_dir, opts_for_run, pabot_args
+            suite_groups, datasources, outs_dir, opts_for_run, pabot_args
         )
     else:
         all_items = _create_execution_items_for_run(
-            suite_names, datasources, outs_dir, options, opts_for_run, pabot_args
+            suite_groups, datasources, outs_dir, options, opts_for_run, pabot_args
         )
     _construct_index_and_completed_index(all_items)
     _construct_last_levels(all_items)
@@ -1661,12 +1672,12 @@ def _construct_index_and_completed_index(all_items):
 
 
 def _create_execution_items_for_run(
-    suite_names, datasources, outs_dir, options, opts_for_run, pabot_args
+    suite_groups, datasources, outs_dir, options, opts_for_run, pabot_args
 ):
     global _NUMBER_OF_ITEMS_TO_BE_EXECUTED
     all_items = []  # type: List[List[QueueItem]]
     _NUMBER_OF_ITEMS_TO_BE_EXECUTED = 0
-    for suite_group in suite_names:
+    for suite_group in suite_groups:
         # TODO: Fix this better
         if (
             options.get("randomize") in ["all", "suites"]
@@ -1700,13 +1711,13 @@ def _create_items(datasources, opts_for_run, outs_dir, pabot_args, suite_group):
 
 
 def _create_execution_items_for_dry_run(
-    suite_names, datasources, outs_dir, opts_for_run, pabot_args
+    suite_groups, datasources, outs_dir, opts_for_run, pabot_args
 ):
     global _NUMBER_OF_ITEMS_TO_BE_EXECUTED
     all_items = []  # type: List[List[QueueItem]]
     _NUMBER_OF_ITEMS_TO_BE_EXECUTED = 0
     processes_count = pabot_args["processes"]
-    for suite_group in suite_names:
+    for suite_group in suite_groups:
         items = _create_items(
             datasources, opts_for_run, outs_dir, pabot_args, suite_group
         )
@@ -1856,23 +1867,15 @@ def main(args=None):
         if _pabotlib_in_use():
             _initialize_queue_index()
         outs_dir = _output_dir(options)
-        suite_names = solve_suite_names(outs_dir, datasources, options, pabot_args)
-        suite_names = shard_suites(suite_names, pabot_args)
+        suite_groups = _group_suites(outs_dir, datasources, options, pabot_args)
         if pabot_args["verbose"]:
             _write("Suite names resolved in %s seconds" % str(time.time() - start_time))
-        ordering = pabot_args.get("ordering")
-        if ordering:
-            suite_names = _preserve_order(suite_names, ordering)
-        if pabot_args["chunk"]:
-            suite_names = _chunked_suite_names(suite_names, pabot_args["processes"])
-        else:
-            suite_names = _group_by_wait(_group_by_groups(suite_names))
-        if not suite_names or suite_names == [[]]:
+        if not suite_groups or suite_groups == [[]]:
             _write("No tests to execute")
             if not options.get("runemptysuite", False):
                 sys.exit(252)
         execution_items = _create_execution_items(
-            suite_names, datasources, outs_dir, options, opts_for_run, pabot_args
+            suite_groups, datasources, outs_dir, options, opts_for_run, pabot_args
         )
         while execution_items:
             items = execution_items.pop(0)
@@ -1889,7 +1892,7 @@ def main(args=None):
             pabot_args,
             options,
             start_time_string,
-            _get_suite_root_name(suite_names),
+            _get_suite_root_name(suite_groups),
         )
         sys.exit(result_code if not _ABNORMAL_EXIT_HAPPENED else 252)
     except Information as i:
@@ -1912,6 +1915,17 @@ def main(args=None):
         _stop_message_writer()
 
 
+def _group_suites(outs_dir, datasources, options, pabot_args):
+    suite_names = solve_suite_names(outs_dir, datasources, options, pabot_args)
+    _verify_depends(suite_names)
+    shard_suites = solve_shard_suites(suite_names, pabot_args)
+    ordered_suites = _preserve_order(shard_suites, pabot_args.get("ordering"))
+    grouped_suites = _chunked_suite_names(ordered_suites, pabot_args["processes"]) if pabot_args["chunk"] \
+        else _group_by_wait(_group_by_groups(ordered_suites))
+    grouped_by_depend = _all_grouped_suites_by_depend(grouped_suites)
+    return grouped_by_depend
+
+
 def _chunked_suite_names(suite_names, processes):
     q, r = divmod(len(suite_names), processes)
     result = []
@@ -1928,5 +1942,48 @@ def _chunked_suite_names(suite_names, processes):
     return [result]
 
 
+def _verify_depends(suite_names):
+    runnable_suites = list(filter(lambda suite: isinstance(suite, RunnableItem), suite_names))
+    suites_with_depends = list(filter(lambda suite: suite.depends, runnable_suites))
+    suites_with_found_dependencies = list(
+        filter(lambda suite: any(runnable_suite.name == suite.depends for runnable_suite in runnable_suites),
+               suites_with_depends))
+    if suites_with_depends != suites_with_found_dependencies:
+        raise Exception("There are unmet dependencies using #DEPENDS")
+    suites_with_circular_dependencies = list(filter(lambda suite: suite.depends == suite.name, suites_with_depends))
+    if suites_with_circular_dependencies:
+        raise Exception("There are suites with circular dependencies using #DEPENDS")
+    grouped_suites = list(filter(lambda suite: isinstance(suite, GroupItem), suite_names))
+    if grouped_suites and suites_with_depends:
+        raise Exception("#DEPENDS and grouped suites are incompatible")
+
+
+def _group_by_depend(suite_names):
+    group_items = list(filter(lambda suite: isinstance(suite, GroupItem), suite_names))
+    runnable_suites = list(filter(lambda suite: isinstance(suite, RunnableItem), suite_names))
+    if group_items or not runnable_suites:
+        return [suite_names]
+    independent_tests = list(filter(lambda suite: not suite.depends, runnable_suites))
+    dependency_tree = [independent_tests]
+    while True:
+        dependent_tests = list(filter(lambda suite: suite.depends, runnable_suites))
+        dependent_on_last_stage = list(filter(
+            lambda suite: any(test_in_tier_before.name == suite.depends for test_in_tier_before in dependency_tree[-1]),
+            dependent_tests))
+        if not dependent_on_last_stage: break
+        dependency_tree += [dependent_on_last_stage]
+    flattened_dependency_tree = sum(dependency_tree, [])
+    if len(flattened_dependency_tree) != len(runnable_suites):
+        raise Exception("There are circular or unmet dependencies using #DEPENDS")
+    return dependency_tree
+
+
+def _all_grouped_suites_by_depend(grouped_suites):
+    grouped_by_depend = []
+    for group_suite in grouped_suites:
+        grouped_by_depend += _group_by_depend(group_suite)
+    return grouped_by_depend
+
+
 if __name__ == "__main__":
     main()
